/* 
 * The Contents of this file are made available subject to the terms of 
 * the GNU Lesser General Public License Version 2.1
 *
 * GNU Lesser General Public License Version 2.1
 * =============================================
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1, as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 * 
 * Copyright (c) 2005 Cedric Bosdonnat
 *
 * All Rights Reserved.
 */
#include <sal/config.h>
#include <com/sun/star/lang/XMultiComponentFactory.hpp>
#include <com/sun/star/lang/XSingleComponentFactory.hpp>
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#include <com/sun/star/registry/XRegistryKey.hpp>
#include <com/sun/star/registry/InvalidRegistryException.hpp>

#include <cppuhelper/factory.hxx>
#include <cppuhelper/implementationentry.hxx>

#include "org/stareditor/comp/exercise/ExerciseImpl.hxx"
#include "org/stareditor/comp/exercise/ServerImpl.hxx"
#include "org/stareditor/comp/exercise/ClientImpl.hxx"
#include "org/stareditor/exercise/Client.hpp"
#include "org/stareditor/exercise/XServer.hpp"
#include "org/stareditor/exercise/XClient.hpp"

using namespace rtl;
using namespace std;
namespace css = com::sun::star;
namespace ose = org::stareditor::exercise;

namespace org { namespace stareditor { namespace comp { namespace exercise {


//-------------------------------------------------------------------
sal_Int32 SAL_CALL ExerciseImpl::run (
     css::uno::Sequence <OUString> const &)  throw (
          css::uno::RuntimeException) 
{
    /*
     * Creates a org::stareditor::exercise::Client instance. Thanks
     * to the new-style services, le result is a reference to the
     * org::stareditor::exercise::XClient interface instead of an Any
     * as it was with old-style services.
     *
     * Note that the UNO/C++ binding of an interface is a 
     *    css::uno::Reference<myInterface>
     *    
     * The create method is the one defined in the idl specification
     * of the service. This method is mapped in the following way:
     *
     * IDL definition:
     *    ctor1 (type1 var1, type2 var2) raises (excpt1, excpt2)
     * C++ method generated:
     *    ctor1 (
     *        Reference<XComponentContext> const & xCtx,
     *        ctype1 var1, ctype2 var2) throw (excpt1, excpt2)
     *
     * Keep in mind that this method is different from the static 
     * create() method of the implementation used for registration.
     */
    css::uno::Reference<ose::XClient> xClient = ose::Client::create(mCtx);
          
    // Make the client asking even more by calling the getMore()
    // method defined in the org::stareditor::exercise::XClient 
    // interface.
    xClient->getMore();
       
    // Like standad C++ programs, return 0 to indicate that the 
    // program ran without any error
    return 0;
}


//===========================================================================
// Exercise Implementation
 
//-------------------------------------------------------------------

/*
 * Implementation of the static create() method defined previously
 */
css::uno::Reference <css::uno::XInterface> SAL_CALL ExerciseImpl::create(
    css::uno::Reference <css::uno::XComponentContext> const & xContext)
{
    return static_cast <cppu::OWeakObject *>(new ExerciseImpl(xContext));
}

/*
 * Implementation of the getSupportedServiceNames() method defined 
 * previously
 */
css::uno::Sequence<OUString> ExerciseImpl::getSupportedServiceNames() 
{
    css::uno::Sequence<OUString> names(1);
    names[0] = OUString(
        RTL_CONSTASCII_USTRINGPARAM(
            "org.stareditor.exercise.Exercise"));
    return names;
}

/*
 * Implementation of the getImplementationName() method defined 
 * previously
 */
OUString ExerciseImpl::getImplementationName()
{
    return OUString( 
        RTL_CONSTASCII_USTRINGPARAM(
            "org.stareditor.comp.exercise.ExerciseImpl") );
}
  
//-------------------------------------------------------------------
/*
 * Implementation of the constructor defined previously. It just 
 * initializes the XComponentContext private member.
 */
ExerciseImpl::ExerciseImpl(
        css::uno::Reference<css::uno::XComponentContext> const & xCtx):
    mCtx(xCtx)
{
}
  
/*
 * Implementation of the previously defined destructor.
 */
ExerciseImpl::~ExerciseImpl()
{
}
  
} // end of exercise

//===========================================================================
// Registry entries definitions for the component

/*
 * This function adds an entry to the regitry key of the component
 * to point out the singletons. Without this key in the registry,
 * the singletons getters won't work.
 */
bool singleton_entries(
    css::uno::Reference<css::registry::XRegistryKey> const & xRegistryKey )
{
    try 
    {
        // Creates the key for the singleton description
        css::uno::Reference<css::registry::XRegistryKey> xKey(
            xRegistryKey->createKey(
                exercise::ServerImpl::getImplementationName() +
                OUString(RTL_CONSTASCII_USTRINGPARAM(
                        "/UNO/SINGLETONS/org.stareditor.exercise.Server"))));
        xKey->setStringValue( 
                exercise::ServerImpl::getImplementationName() );
        return true;
    }
    catch (css::registry::InvalidRegistryException & exc) 
    {
        OSL_ENSURE( 0, rtl::OUStringToOString(
                      exc.Message, RTL_TEXTENCODING_UTF8 ).getStr() );
        return false;
    }
}

/*
 * Structure describing the services implemented in this component. 
 * Be aware that this do not describe the singletons, look at the
 * singleton_entries() and component_writeInfo() methods for 
 * singletons
 */
static struct cppu::ImplementationEntry entries [] =
{
  // Entry describing the Exercise implementation
  {
      exercise::ExerciseImpl::create, 
      exercise::ExerciseImpl::getImplementationName,
      exercise::ExerciseImpl::getSupportedServiceNames,
      cppu::createSingleComponentFactory,
      0, 0
  },
  
  // Entry describing the Client implementation
  {
      exercise::ClientImpl::create,
      exercise::ClientImpl::getImplementationName,
      exercise::ClientImpl::getSupportedServiceNames,
      cppu::createSingleComponentFactory,
       0, 0
  },
  
  // Entry describing the Server implementation
  {
      exercise::ServerImpl::create,
      exercise::ServerImpl::getImplementationName,
      exercise::ServerImpl::getSupportedServiceNames,
      cppu::createSingleComponentFactory,
      0, 0
  },

  // The structure has to be ended by a NULL filled entry
  { 0, 0, 0, 0, 0, 0 }
};

} // end of comp
} // end of stareditor
} // end of org

//===========================================================================
// Component Registration exported C functions. These methods are
// used by the regcomp tool.

extern "C" 
{
    /*
     * Function copied from the Developer's Guide, Chap 4.
     */
    void SAL_CALL component_getImplementationEnvironment(
        sal_Char const ** ppEnvTypeName, uno_Environment ** ppEnv )
    {
          *ppEnvTypeName = CPPU_CURRENT_LANGUAGE_BINDING_NAME;
    }

    /*
     * Function copied from the Developer's Guide, Chap 4.
     */
    void * SAL_CALL component_getFactory( 
        char const * implName, void * xMgr, void * registryKey)
    {
      return cppu::component_getFactoryHelper(
          implName, xMgr, registryKey, org::stareditor::comp::entries);
    }

    /*
     * Function writing the component informations. Note that this is
     * a function copied from the Developer's Guide with a small
     * change to add the singleton to the registry key.
     */
    sal_Bool SAL_CALL component_writeInfo( void * xMgr, 
        css::registry::XRegistryKey * xRegistry )
    {
      return cppu::component_writeInfoHelper(
          xMgr, xRegistry, org::stareditor::comp::entries) && 
              org::stareditor::comp::singleton_entries(xRegistry);
    }
}
