/* 
 * The Contents of this file are made available subject to the terms of 
 * the GNU Lesser General Public License Version 2.1
 *
 * GNU Lesser General Public License Version 2.1
 * =============================================
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1, as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 * 
 * Copyright (c) 2005 Cedric Bosdonnat
 *
 * All Rights Reserved.
 */
#include <sal/config.h>
#include <com/sun/star/lang/XMain.hpp>
#include <com/sun/star/uno/XComponentContext.hpp>
#include <com/sun/star/uno/Sequence.hxx>
#include <com/sun/star/uno/RuntimeException.hpp>
#include <com/sun/star/uno/Reference.hxx>

#include <cppuhelper/implbase1.hxx>

#include <rtl/string.hxx>

namespace css = com::sun::star;

namespace org { namespace stareditor { namespace comp { namespace exercise {

  // Implementation for the org.stareditor.exercise.Exercise service
  // launching the application.
  //
  // EXPL1: Why XMain ?
  //     In a C++ implementation, the service which launch the
  //     application has to implement the com::sun::star::lang::XMain
  //     interface. The principle is very easy to understand: this
  //     interface defines a run() method that is run after 
  //     bootstrapping the URE. This method plays the same role as 
  //     the main(argc, argv) function in standard C++.
  //
  // EXPL2: Why this class inherits from cppu::WeakImplHelper1 ?
  //     Normally, you should implement every method described in
  //     each parent interface of your service. Here a helper is
  //     used in order to avoid rewriting all these common methods.
  //     This way, there are only the XMain method described here.
  //     The cppu namespace defines a lot of helpers like this
  //     one, however it will be the most commonly used. The template
  //     form WeakImplHelper1<myInterface> means: "This class 
  //     implements the myType interface". The '1' means that this
  //     service is implementing only one interface, however the
  //     new-style services can't implement more than one ;)
  //     
  class ExerciseImpl: public cppu::WeakImplHelper1 <css::lang::XMain> 
  {

    public:

      /*
       * Constructor for this main implementation. The explicit keyword
       * is recommended when there is only one argument in the 
       * constructor to avoid problems like:
       *
       *   css::uno::ExerciseImpl aExercise = aContext;
       */
      explicit ExerciseImpl (
           css::uno::Reference <css::uno::XComponentContext> const & xCtx);
     
      /*
       * The method that replaces the standard C++ main() function.
       */
      sal_Int32 SAL_CALL run (
              css::uno::Sequence <rtl::OUString> const &)  throw (
                  css::uno::RuntimeException); 

      /*
       * Static creation method of the ExerciseImpl class used by the 
       * component registration methods
       */
      static css::uno::Reference <css::uno::XInterface> SAL_CALL create(
          css::uno::Reference <css::uno::XComponentContext> const & xContext);
      
      /*
       * Method returning the supported service names for this 
       * implementation class.
       */
      static css::uno::Sequence<rtl::OUString> getSupportedServiceNames(); 
      
      /*
       * Method returning the name of the implementation. 
       * This is used by the registration methods.
       */
      static rtl::OUString getImplementationName();

    private:
      
      /*
       * Private destructor for the main service
       */
      ~ExerciseImpl();

      /*
       * Private member to store the Component Context, used for the
       * other services creation.
       */
      css::uno::Reference <css::uno::XComponentContext> mCtx;
  };

} } } } // end of all the namespaces
